@@ -16,8 +16,10 @@ void extension_build_params(PyObject *params_py, params *params) {
         element = PyIter_Next(iterator);
         if(element == NULL) { break; }
 
-        key = PyDict_GetItemString(element, "key");
-        value = PyDict_GetItemString(element, "value");
+        params->length++;
+
+        key = PySequence_Fast_GET_ITEM(element, 0);
+        value = PySequence_Fast_GET_ITEM(element, 1);
 
 #if PY_MAJOR_VERSION >= 3
         key = PyUnicode_EncodeFSDefault(key);
@@ -26,30 +28,60 @@ void extension_build_params(PyObject *params_py, params *params) {
         key_s = PyString_AsString(key);
 #endif
 
-        params->length++;
-        params->params[index].key = key_s;
+        memcpy(
+            params->params[index].key,
+            key_s,
+            strlen(key_s) + 1
+        );
+
+#if PY_MAJOR_VERSION >= 3
+        Py_DECREF(key);
+#endif
 
         if(PyBool_Check(value)) {
             value_b = PyBool_Check(value);
             params->params[index].value.boolean = value_b;
-        } else if(PyInt_Check(value)) {
+        }
+#if PY_MAJOR_VERSION < 3
+        else if(PyInt_Check(value)) {
             value_i = PyInt_AsLong(value);
             params->params[index].value.integer = value_i;
+        }
+#endif
+        else if(PyLong_Check(value)) {
+            value_i = PyLong_AsLong(value);
+            params->params[index].value.integer = value_i;
         } else if(PyFloat_Check(value)) {
             value_f = (float) PyFloat_AsDouble(value);
             params->params[index].value.decimal = value_f;
+#if PY_MAJOR_VERSION >= 3
+        } else if(PyUnicode_Check(value)) {
+#else
         } else if(PyString_Check(value)) {
+#endif
 #if PY_MAJOR_VERSION >= 3
             value = PyUnicode_EncodeFSDefault(value);
             value_s = PyBytes_AsString(value);
 #else
             value_s = PyString_AsString(value);
 #endif
-            params->params[index].value.string = value_s;
+            memcpy(
+                params->params[index].value.string,
+                value_s,
+                strlen(value_s) + 1
+            );
+
+#if PY_MAJOR_VERSION >= 3
+            Py_DECREF(value);
+#endif
         }
 
+        Py_DECREF(element);
+
         index++;
     }
+
+    Py_DECREF(iterator);
 };
 
 PyObject *extension_register(PyObject *self, PyObject *args) {
@@ -89,6 +121,7 @@ PyObject *extension_blend_images(PyObject *self, PyObject *args, PyObject *kwarg
         args,
         kwargs,
         "sss|sO",
+        kwlist,
         &bottom_path,
         &top_path,
         &target_path,
@@ -132,7 +165,7 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     char *bottom_path, *top_path, *target_path, *algorithm = NULL;
     struct pcv_image bottom, top;
     PyObject *paths, *iterator, *iteratorAlgorithms, *element, *first, *second,
-        *is_inline, *algorithms, *algorithm_o, *params_py;
+        *is_inline, *algorithms, *algorithm_o, *algorithm_so, *params_py;
     Py_ssize_t size, algorithms_size;
     param values[32];
     params params = { 0, values };
@@ -147,7 +180,7 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     };
 
 #if PY_MAJOR_VERSION >= 3
-    PyObject *encoded;
+    PyObject *encoded, *algorithm_e;
 #endif
 
     paths = NULL;
@@ -191,12 +224,12 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
 
     /* retrieves the size of the paths sequence that has been provided
     this is relevant to determine the kind of operation to be performed */
-    size = PyList_Size(paths);
+    size = PySequence_Size(paths);
 
     /* in case the use algorithms list strategy is used a small set of
     verifications must be performed */
     if(use_algorithms) {
-        algorithms_size = PyList_Size(algorithms);
+        algorithms_size = PySequence_Size(algorithms);
         if(algorithms_size != size - 1) {
             abort_("[extension_blend_multiple] Invalid algorithms list size");
         }
@@ -206,7 +239,7 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     is just one then a simple re-write operation should be
     performed, read from provided path and write to target path */
     if(size == 1) {
-        first = PyList_GetItem(paths, 0);
+        first = PySequence_Fast_GET_ITEM(paths, 0);
 
 #if PY_MAJOR_VERSION >= 3
         first = PyUnicode_EncodeFSDefault(first);
@@ -228,6 +261,9 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
             Py_BLOCK_THREADS Py_RETURN_NONE
         );
         Py_END_ALLOW_THREADS;
+#if PY_MAJOR_VERSION >= 3
+        Py_DECREF(first);
+#endif
         Py_RETURN_NONE;
     }
 
@@ -240,31 +276,37 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     target algorithm for the current composition is retrieved
     from the algorithms list*/
     if(use_algorithms) {
-        algorithm_o = PyList_GetItem(algorithms, 0);
+        algorithm_o = PySequence_Fast_GET_ITEM(algorithms, 0);
 
-        /* verifies if the provided algorithm value is a tuple
+        /* verifies if the provided algorithm value is a sequence
         and if that's the case (parameters are provided for its
         execution) then loads them properly */
-        if(PyTuple_Check(algorithm_o)) {
-            params_py = PyTuple_GetItem(algorithm_o, 1);
-            algorithm_o = PyTuple_GetItem(algorithm_o, 0);
+        if(PyList_Check(algorithm_o) || PyTuple_Check(algorithm_o)) {
+            params_py = PySequence_Fast_GET_ITEM(algorithm_o, 1);
+            algorithm_o = PySequence_Fast_GET_ITEM(algorithm_o, 0);
             extension_build_params(params_py, &params);
         }
 
 #if PY_MAJOR_VERSION >= 3
-        algorithm_o = PyUnicode_EncodeFSDefault(algorithm_o);
-        algorithm = PyBytes_AsString(algorithm_o);
+        algorithm_e = PyUnicode_EncodeFSDefault(algorithm_o);
+        algorithm = PyBytes_AsString(algorithm_e);
 #else
         algorithm = PyString_AsString(algorithm_o);
 #endif
+
         source_over = strcmp(algorithm, "source_over") == 0;
         destination_over = strcmp(algorithm, "destination_over") == 0;
+    } else {
+        algorithm_o = NULL;
+#if PY_MAJOR_VERSION >= 3
+        algorithm_e = NULL;
+#endif
     }
 
     /* retrieves the first two elements from the list to serve
     as the initial two images to be used in the first composition */
-    first = PyList_GetItem(paths, 0);
-    second = PyList_GetItem(paths, 1);
+    first = PySequence_Fast_GET_ITEM(paths, 0);
+    second = PySequence_Fast_GET_ITEM(paths, 1);
 
 #if PY_MAJOR_VERSION >= 3
     first = PyUnicode_EncodeFSDefault(first);
@@ -317,18 +359,19 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
 #if PY_MAJOR_VERSION >= 3
     Py_DECREF(first);
     Py_DECREF(second);
+    if(use_algorithms) { Py_DECREF(algorithm_e); }
 #endif
 
     iterator = PyObject_GetIter(paths);
 
     /* increments the iterator by both the top and the bottom
     paths, as they have already been processed */
-    PyIter_Next(iterator);
-    PyIter_Next(iterator);
+    Py_DECREF(PyIter_Next(iterator));
+    Py_DECREF(PyIter_Next(iterator));
 
     if(use_algorithms) {
         iteratorAlgorithms = PyObject_GetIter(algorithms);
-        PyIter_Next(iteratorAlgorithms);
+        Py_DECREF(PyIter_Next(iteratorAlgorithms));
     }
 
     /* iterates continuously until the iterator is exhausted and
@@ -336,6 +379,7 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     while(TRUE) {
         element = PyIter_Next(iterator);
         if(element == NULL) { break; }
+
 #if PY_MAJOR_VERSION >= 3
         encoded = PyUnicode_EncodeFSDefault(element);
         top_path = PyBytes_AsString(encoded);
@@ -346,23 +390,32 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
         if(use_algorithms) {
             algorithm_o = PyIter_Next(iteratorAlgorithms);
 
-            /* verifies if the provided algorithm value is a tuple
+            /* verifies if the provided algorithm value is a sequence
             and if that's the case (parameters are provided for its
             execution) then loads them properly */
-            if(PyTuple_Check(algorithm_o)) {
-                params_py = PyTuple_GetItem(algorithm_o, 1);
-                algorithm_o = PyTuple_GetItem(algorithm_o, 0);
+            if(PyList_Check(algorithm_o) || PyTuple_Check(algorithm_o)) {
+                algorithm_so = PySequence_Fast_GET_ITEM(algorithm_o, 0);
+                params_py = PySequence_Fast_GET_ITEM(algorithm_o, 1);
+
                 extension_build_params(params_py, &params);
+            } else {
+                algorithm_so = algorithm_o;
             }
 
 #if PY_MAJOR_VERSION >= 3
-            algorithm_o = PyUnicode_EncodeFSDefault(algorithm_o);
-            algorithm = PyBytes_AsString(algorithm_o);
+            algorithm_e = PyUnicode_EncodeFSDefault(algorithm_so);
+            algorithm = PyBytes_AsString(algorithm_e);
 #else
-            algorithm = PyString_AsString(algorithm_o);
+            algorithm = PyString_AsString(algorithm_so);
 #endif
+
             source_over = strcmp(algorithm, "source_over") == 0;
             destination_over = strcmp(algorithm, "destination_over") == 0;
+        } else {
+            algorithm_o = NULL;
+#if PY_MAJOR_VERSION >= 3
+            algorithm_e = NULL;
+#endif
         }
 
         Py_BEGIN_ALLOW_THREADS;
@@ -395,8 +448,10 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
         VALIDATE_A(release_image(&top), Py_BLOCK_THREADS Py_RETURN_NONE);
         Py_END_ALLOW_THREADS;
         Py_DECREF(element);
+        if(use_algorithms) { Py_DECREF(algorithm_o); }
 #if PY_MAJOR_VERSION >= 3
         Py_DECREF(encoded);
+        if(use_algorithms) { Py_DECREF(algorithm_e); }
 #endif
     }
 
